==> pymetrick-0.41.zip: pymetrick-0.41/pymetrick/image.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Modulo para el tratamiento de imagenes"""

__author__ = 'Fco.Javier Tamarit V <javtamvi@gmail.com>'
__copyright__ = "Copyright (C) 2012 PYMETRICK.ORG"
__date__ = '2012-09-21'
__version__ = '0.4'
__license__ = 'Licencia GNU/GPLv2'
__credits__ = ''
__text__ = 'Utilidades con imagenes'
__file__ = 'image.py'

#--- CHANGES ------------------------------------------------------------------
# 2012-09-21 v0.01 PL: - First version
# 2016-08-10 v0.02 PL: - bug fixes


# Nota:
# En linux antes de instalar Pillow : sudo yum install python-devel libjpeg-devel zlib-devel
#                                     pip install Pillow     o    easy_install Pillow
from PIL import Image
import base64
import urllib2
import os, sys, string
import hashlib
import logging

# Version PY
PY2 = True if sys.version[:1] == '2' else False;
PY3 = True if sys.version[:1] == '3' else False;
IMAGE_ALLOWED_EXTENSIONS = ['.bmp', '.png', '.jpg', 'jpeg', '.gif']

# Comprobar codigo en desarrollo
DEBUG = False
#Tratamiento de errores en LOG
__name__ = "image"
sys.stdout = sys.stderr

if DEBUG:
    logPath = os.path.abspath(os.curdir)
    logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
    LOG = logging.getLogger(__name__)
    hdlr = logging.FileHandler(logPath+__name__+'.log')
    formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
    hdlr.setFormatter(formatter)
    LOG.addHandler(hdlr)
    LOG.info("Session modo local INICIO")
else:
    logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
    LOG = logging.getLogger(__name__)

def image_size(__path__):
    '''Comprueba la dimension de la imagen'''
    try:
        #__path__ = __path__.lower()
        if os.path.exists(__path__):
            if __path__ and (__path__[-4:].lower() in IMAGE_ALLOWED_EXTENSIONS):
                im = Image.open(__path__)
                width, height = im.size
                return width, height
	else:
            return ()
    except Exception as e:
        LOG.error('Error image.image_size %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

def image_resize(__path__,__factor__=1,__resized_path__=''):
    '''Convierte fichero de imagen a un tamaño diferente, ademas
       es posible cambiar el tipo de imagen de png, jpeg o gif a otro
       cuando se renombra el fichero de imagen resultante.
    '''
    try:
        #__path__ = __path__.lower()
        if os.path.exists(__path__):
            if __path__ and (__path__[-4:].lower() in IMAGE_ALLOWED_EXTENSIONS):
                im = Image.open(__path__)
                width, height = im.size
                max_dim = max(im.size)
                if __factor__:
                    i = float(__factor__)
                    side = int(max_dim*i)
                    resized_image = im.resize((side, side), Image.ANTIALIAS)
                    if __resized_path__ and (__resized_path__[-4:].lower() in IMAGE_ALLOWED_EXTENSIONS):
                        resized_image.save(__resized_path__)
                        return __resized_path__
		    else:
			return ''
		else:
		    return ''
	    else:
		return ''
	else:
            return ''
    except Exception as e:
        LOG.error('Error image.image_resize %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

def image2html(__path__):
    '''Devuelve una imagen en base64 como un string de forma que
       se utiliza como imagen embebida en css o html

       El formato para html es :
       <img alt="explicacion" width="57" height="57" src="data:image/png;base64,_string_"/>
       las opciones de data son image/png, image/jpeg, image/gif

       en CSS :
       #header-bd{
       width:12px;
       height:38px;
       background-image:url("data:image/jpeg;base64,_string_");
       background-repeat:no-repeat;
       }
       
       @return
       dict con datos de imagen
       with 
       height
       type = "data:image/?;base64"
       image = imagen en formato base64
    '''
    try:
        #__path__ = __path__.lower()
        if os.path.exists(__path__):
            if __path__ and (__path__[-4:].lower() in IMAGE_ALLOWED_EXTENSIONS):
                '''Dimension de imagen'''
                im_size = image_size(__path__)
                '''Tipo de imagen'''
                im_ext = __path__[-3:]
                '''Cadena de imagen en base64'''
                im_img = image2base64(__path__)

                im_data = dict()
                im_data['width'] = im_size[0]
                im_data['height'] = im_size[1]
                im_data['type'] = 'data:image/%s;base64,' % im_ext
                im_data['image'] = im_img
                return im_data
	    else:
		return {}
	else:
	    return {}
    except Exception as e:
        LOG.error('Error image.image2html %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

def image2base64(__path__, blocksize=65536):
    '''Convierte un fichero de imagen en base64 extendido
       @Parameters
           __path__ : direccion donde se encuentra el fichero de imagen
       @Return
           file_name$string_base64
    '''
    try:
        #__path__ = __path__.lower()
        if os.path.exists(__path__):
            if __path__ and (__path__[-4:].lower() in IMAGE_ALLOWED_EXTENSIONS):
                with open(__path__, "rb") as img_file:
                    # codificar imagen como base64
                    img_string = ''
                    for block in iter(lambda: img_file.read(),b""):
                        img_string = base64.b64encode(block)
                    return '%s' % (img_string)
		img_file.close()
            else:
                return ''
        else:
            return ''
    except Exception as e:
        LOG.error('Error image.image2base64 %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

def image_download(url, fname):
    """Descarga una imagen desde una url y se puede renombrar.
       download('http://nadadenada.com/coche.png',
       '/img/coche_2.png')
    """
    try:
        furl = urllib2.urlopen(url)
        f = file(fname,'wb')
        f.write(furl.read())
        f.close()
    except Exception as e:
        LOG.error('Error image.image_download %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

'''* Convertir imagen a gris   im = im.convert ("L")
   * Muestra imagen con im.show()
   * Funcionas basicas
   im.format, im.size,   im.mode
    JPEG      (384, 423) RGB
   * Obetener el valor de un pixel
   im.getpixel((10,20))
   (254, 254, 254)
   '''

def image2checksums(__path__, blocksize=65536):
    """obtener el hash de una imagen para identificarla de forma unica
       esta firma de imagen es util para comparar si la imagen es la misma o diferente a otras
    """
    try:
        #__path__ = __path__.lower()
        im_hash = ''
        if __path__ and (__path__[-4:].lower() in IMAGE_ALLOWED_EXTENSIONS):
	    hash = hashlib.md5()
	    with open(__path__, "rb") as img:
		for block in iter(lambda: img.read(blocksize), b""):
		    hash.update(block)
	    return hash.hexdigest()
        else:
	    return ''
    except Exception as e:
        LOG.error('Error image.image2checksums %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

"""
   Metadatos de imagenes   hex(ord(n))
   jpeg     0xFF, 0xD8 	Start Of Image            0xFF, 0xD9 End Of Image
"""

if __name__ == "__main__":
    if len(sys.argv) < 2:
        sys.stderr.write("for help use -h o --help")
    elif sys.argv[1]=='-h' or sys.argv[1]=='--help':
        print u'''Utilidades con imagenes
        '''
        print u'''image_size(__path__) Comprueba las dimensiones de la imagen.\n'''
        print u'''image_resize(__path__,__factor__=1,__resized_path__='') Convierte fichero de imagen a un tamaño diferente, ademas es posible cambiar el tipo de imagen de png, jpeg o gif a otro cuando se renombra el fichero de imagen resultante.\n'''
        print u'''image2html('logo.png') Devuelve una imagen en base64 como un string de forma que se utiliza como imagen embebida en css o html.\n'''
        print u'''image2base64('logo.png') Devuelve una imagen en base64 extendido  ( file_name$string_base64).\n'''
        print u'''image_download('http://nadadenada.com/coche.png', '/img/coche_2.png') Descarga una imagen desde una url y permite renombrar el fichero.\n'''
        print u'''image2checksums('logo.png') Devuelve un hash como un string de forma que se puede comparar con otras imagenes \n'''

==> pymetrick-0.42.zip: pymetrick-0.42/pymetrick/image.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Modulo para el tratamiento de imagenes"""

from pymetrick import version
__author__ = version.__author__
__copyright__ = version.__copyright__
__license__ = version.__license__
__version__ = version.__version__
__date__ = '2012-09-21'
__credits__ = ''
__text__ = 'Utilidades con imagenes'
__file__ = 'image.py'

#--- CHANGES ------------------------------------------------------------------
# 2012-09-21 v0.01 PL: - First version
# 2016-08-10 v0.02 PL: - bug fixes

# Nota:
# En linux antes de instalar Pillow : sudo yum install python-devel libjpeg-devel zlib-devel
#                                     pip install Pillow     o    easy_install Pillow
from PIL import Image
import base64
import os, sys, string
import hashlib
import math, operator
import logging

try:
    import urllib.request                # python 3
except:
    import urllib2                       # python 2

# Version PY
PY2 = True if sys.version[:1] == '2' else False;
PY3 = True if sys.version[:1] == '3' else False;
IMAGE_ALLOWED_EXTENSIONS = ['.bmp', '.png', '.jpg', 'jpeg', '.gif']

# Comprobar codigo en desarrollo
DEBUG = False
#Tratamiento de errores en LOG
__name__ = "image"
sys.stdout = sys.stderr

if DEBUG:
    logPath = os.path.abspath(os.curdir)
    logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
    LOG = logging.getLogger(__name__)
    hdlr = logging.FileHandler(logPath+os.sep+__name__+'.log')
    formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
    hdlr.setFormatter(formatter)
    LOG.addHandler(hdlr)
    LOG.info("Session modo local INICIO")
else:
    logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
    LOG = logging.getLogger(__name__)

def image_size(__path__):
    '''Comprueba la dimension de la imagen'''
    try:
        __size__=list()
        #__path__ = __path__.lower()
        if os.path.exists(__path__):
            if __path__ and (__path__[-4:].lower() in IMAGE_ALLOWED_EXTENSIONS):
                im = Image.open(__path__)
                #width, height = im.size
                __size__=im.size
        return __size__
    except Exception as e:
        LOG.error('Error image.image_size %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

def image_resize(__path__,__factor__=1,__resized_path__=''):
    '''Convierte fichero de imagen a un tamaño diferente, ademas
       es posible cambiar el tipo de imagen de png, jpeg o gif a otro
       cuando se renombra el fichero de imagen resultante.
    '''
    try:
        #__path__ = __path__.lower()
        if os.path.exists(__path__):
            if __path__ and (__path__[-4:].lower() in IMAGE_ALLOWED_EXTENSIONS):
                im = Image.open(__path__)
                width, height = im.size
                max_dim = max(im.size)
                if __factor__:
                    i = float(__factor__)
                    side = int(max_dim*i)
                    resized_image = im.resize((side, side), Image.ANTIALIAS)
                    if __resized_path__ and (__resized_path__[-4:].lower() in IMAGE_ALLOWED_EXTENSIONS):
                        resized_image.save(__resized_path__)
                        return __resized_path__
                    else:
                        return ''
                else:
                    return ''
            else:
                return ''
        else:
            return ''
    except Exception as e:
        LOG.error('Error image.image_resize %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

def image2html(__path__):
    '''Devuelve una imagen en base64 como un string de forma que
       se utiliza como imagen embebida en css o html

       El formato para html es :
       <img alt="explicacion" width="57" height="57" src="data:image/png;base64,_string_"/>
       las opciones de data son image/png, image/jpeg, image/gif

       en CSS :
       #header-bd{
       width:12px;
       height:38px;
       background-image:url("data:image/jpeg;base64,_string_");
       background-repeat:no-repeat;
       }

       @return
       dict con datos de imagen
       with
       height
       type = "data:image/?;base64"
       image = imagen en formato base64
    '''
    try:
        #__path__ = __path__.lower()
        im_data = dict()
        if os.path.exists(__path__):
            if __path__ and (__path__[-4:].lower() in IMAGE_ALLOWED_EXTENSIONS):
                '''Dimension de imagen'''
                im_size = image_size(__path__)
                '''Tipo de imagen'''
                im_ext = __path__[-3:]
                '''Cadena de imagen en base64'''
                im_img = image2base64(__path__)

                im_data['width'] = im_size[0]
                im_data['height'] = im_size[1]
                im_data['type'] = 'data:image/%s;base64,' % im_ext
                im_data['image'] = im_img
                return im_data
        return im_data
    except Exception as e:
        LOG.error('Error image.image2html %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

def image2base64(__path__, blocksize=65536):
    '''Convierte un fichero de imagen en base64 extendido
       @Parameters
           __path__ : direccion donde se encuentra el fichero de imagen
       @Return
           file_name$string_base64
    '''
    try:
        #__path__ = __path__.lower()
        if os.path.exists(__path__):
            if __path__ and (__path__[-4:].lower() in IMAGE_ALLOWED_EXTENSIONS):
                with open(__path__, "rb") as img_file:
                    # codificar imagen como base64
                    img_string = ''
                    for block in iter(lambda: img_file.read(),b""):
                        img_string = base64.b64encode(block)
                    return '%s' % (img_string)
                img_file.close()
            else:
                return ''
        else:
            return ''
    except Exception as e:
        LOG.error('Error image.image2base64 %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

def image_download(url, fname):
    """Descarga una imagen desde una url y se puede renombrar.
       download('http://nadadenada.com/coche.png',
       '/img/coche_2.png')
    """
    try:
        if PY2:
            furl = urllib2.urlopen(url)
        elif PY3:
            furl = urllib.request.urlopen(request)
        f = file(fname,'wb')
        f.write(furl.read())
        f.close()
    except Exception as e:
        LOG.error('Error image.image_download %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

'''* Convertir imagen a gris   im = im.convert ("L")
   * Muestra imagen con im.show()
   * Funcionas basicas
   im.format, im.size,   im.mode
    JPEG      (384, 423) RGB
   * Obetener el valor de un pixel
   im.getpixel((10,20))
   (254, 254, 254)
   '''

def image_checksums(__path__, blocksize=65536):
    """obtener el hash de una imagen para identificarla de forma unica
       esta firma de imagen es util para comparar si la imagen es la misma o diferente a otras
    """
    try:
        #__path__ = __path__.lower()
        im_hash = ''
        if __path__ and (__path__[-4:].lower() in IMAGE_ALLOWED_EXTENSIONS):
            hash = hashlib.md5()
            with open(__path__, "rb") as img:
                for block in iter(lambda: img.read(blocksize), b''):
                    hash.update(block)
            return hash.hexdigest()
        else:
            return ''
    except Exception as e:
        LOG.error('Error image.image2checksums %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

def image_compare(file1,file2):
    """Comprueba si las imagenes son iguales o diferentes
       return 0 => iguales
       return <>0 => diferentes
    """
    i1 = Image.open(file1)
    i2 = Image.open(file2)
    if i1.mode == i2.mode and i1.size == i2.size:
        h1 = i1.histogram()
        h2 = i2.histogram()
        rms = math.sqrt(reduce(operator.add,map(lambda a,b: (a-b)**2, h1, h2))/len(h1))
        return rms
    else:
        return 1000.1

"""
   Metadatos de imagenes   hex(ord(n))
   jpeg     0xFF, 0xD8  Start Of Image            0xFF, 0xD9 End Of Image
"""

if __name__ == "__main__":
    if len(sys.argv) < 2:
        sys.stderr.write("for help use -h o --help")
    elif sys.argv[1]=='-h' or sys.argv[1]=='--help':
        print (u'''version {0}'''.format( __version__))
        print (u'''Utilidades con imagenes''')
        print (u'''image_size(__path__) Comprueba las dimensiones de la imagen.\n''')
        print (u'''image_resize(__path__,__factor__=1,__resized_path__='') Convierte fichero de imagen a un tamaño diferente, ademas es posible cambiar el tipo de imagen de png, jpeg o gif a otro cuando se renombra el fichero de imagen resultante.\n''')
        print (u'''image2html('logo.png') Devuelve una imagen en base64 como un string de forma que se utiliza como imagen embebida en css o html.\n''')
        print (u'''image2base64('logo.png') Devuelve una imagen en base64 extendido  ( file_name$string_base64).\n''')
        print (u'''image_download('http://nadadenada.com/coche.png', '/img/coche_2.png') Descarga una imagen desde una url y permite renombrar el fichero.\n''')
        print (u'''image2checksums('logo.png') Devuelve un hash como un string de forma que se puede comparar con otras imagenes \n''')
